"""
아카이브 관련 명령어 모듈
"""

import base64
import json
from datetime import datetime
from typing import Dict, List, Optional, Tuple

import rich_click as click
import yaml
from rich.panel import Panel
from rich.progress import Progress, SpinnerColumn, TextColumn

from fown.core.models.config import Label
from fown.core.services.github import LabelService
from fown.core.utils.file_io import console, make_github_api_request


def get_github_username() -> Optional[str]:
    """현재 인증된 GitHub 사용자 이름 가져오기"""
    try:
        user_data = make_github_api_request("GET", "user")
        # 명시적 타입 체크 추가
        if isinstance(user_data, dict):
            return user_data.get("login")
        return None
    except SystemExit:
        return None


def get_user_repositories() -> List[Dict]:
    """사용자의 모든 레포지토리 목록 가져오기"""
    try:
        repos = make_github_api_request(
            "GET", "user/repos", params={"per_page": 100, "type": "owner"}
        )
        # 명시적 타입 체크 추가
        return repos if isinstance(repos, list) else []
    except SystemExit:
        return []


def get_available_repo_name(base_name: str, existing_repos: Optional[List[Dict]] = None) -> str:
    """사용 가능한 레포지토리 이름 찾기"""
    console.print("[info]사용 가능한 레포지토리 이름 확인 중...[/]")
    if existing_repos is None:
        existing_repos = get_user_repositories()
    existing_repo_names = {repo["name"] for repo in existing_repos}

    for i in range(10):
        suffix = "" if i == 0 else str(i)
        repo_name = f"{base_name}{suffix}"
        if repo_name not in existing_repo_names:
            console.print(f"[info]사용 가능한 레포지토리 이름: [bold]{repo_name}[/][/]")
            return repo_name

    import random

    repo_name = f"{base_name}{random.randint(10, 99)}"
    console.print(f"[info]사용 가능한 레포지토리 이름: [bold]{repo_name}[/][/]")
    return repo_name


def create_archive_repo(repo_name: str, description: str, is_public: bool = False) -> bool:
    """GitHub에 아카이브 레포지토리 생성"""
    try:
        data = {
            "name": repo_name,
            "description": description,
            "private": not is_public,
        }
        make_github_api_request("POST", "user/repos", data=data)
        visibility = "public" if is_public else "private"
        console.print(f"[success]✓[/] Created {visibility} repository: [bold]{repo_name}[/]")
        return True
    except SystemExit:
        console.print(f"[error]레포지토리 '{repo_name}' 생성 실패[/]")
        return False


def create_file_in_repo(owner: str, repo_name: str, file_path: str, content: str, message: str):
    """Helper to create a single file in a repository"""
    endpoint = f"repos/{owner}/{repo_name}/contents/{file_path}"
    data = {
        "message": message,
        "content": base64.b64encode(content.encode("utf-8")).decode("utf-8"),
    }
    make_github_api_request("PUT", endpoint, data=data)


def create_fown_config_files(
    repo_owner: str, repo_name: str, labels: List[Label], is_default: bool = True
) -> bool:
    """아카이브 레포지토리에 설정 파일들 생성"""
    try:
        # 1. README.md
        readme_content = (
            f"# {repo_name}\n\n"
            "이 레포지토리는 Fown에서 설정 및 관리를 위한 아카이브 레포지토리입니다.\n"
            f"생성일: {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}\n\n"
        )
        if is_default:
            readme_content += "**이 레포지토리는 기본 설정 레포지토리입니다.**\n\n"
        readme_content += (
            "## 포함된 설정\n\n"
            "- `.fown/config.yml`: 아카이브 레포지토리의 설정 파일\n"
            "- `labels/default_labels.json`: 레이블 템플릿\n"
            "- `scripts/hello_world.sh`: 예제 스크립트\n"
        )
        create_file_in_repo(
            repo_owner, repo_name, "README.md", readme_content, "Initial commit: Add README.md"
        )

        # 2. .fown/config.yml
        config_data = {
            "default_repository": is_default,
            "created_at": datetime.now().isoformat(),
        }
        config_content = yaml.dump(config_data, default_flow_style=False, allow_unicode=True)
        create_file_in_repo(
            repo_owner, repo_name, ".fown/config.yml", config_content, "Create fown config"
        )

        # 3. labels/default_labels.json
        labels_content = json.dumps(
            [label.to_dict() for label in labels], ensure_ascii=False, indent=2
        )
        create_file_in_repo(
            repo_owner,
            repo_name,
            "labels/default_labels.json",
            labels_content,
            "Add default labels",
        )

        # 4. scripts/hello_world.sh
        script_content = "#!/bin/bash\necho 'Hello, World!'\n"
        create_file_in_repo(
            repo_owner, repo_name, "scripts/hello_world.sh", script_content, "Add example script"
        )

        return True
    except SystemExit as e:
        console.print(f"[error]설정 파일 생성 실패:[/] {e}")
        return False


def check_existing_default_repo(
    username: str, base_name: str, existing_repos: Optional[List[Dict]] = None
) -> Tuple[bool, Optional[str]]:
    """기본 아카이브 레포지토리 확인"""
    console.print("[info]기존 기본 레포지토리 검사 중...[/]")
    if existing_repos is None:
        existing_repos = get_user_repositories()
    existing_repo_names = {repo["name"] for repo in existing_repos}

    for i in range(10):
        suffix = "" if i == 0 else str(i)
        repo_name = f"{base_name}{suffix}"
        if repo_name not in existing_repo_names:
            continue

        console.print(f"[info]레포지토리 [bold]{repo_name}[/] 발견, 설정 확인 중...[/]")
        try:
            endpoint = f"repos/{username}/{repo_name}/contents/.fown/config.yml"
            config_data = make_github_api_request("GET", endpoint)
            # 명시적 타입 체크 추가
            if isinstance(config_data, dict) and "content" in config_data:
                content = base64.b64decode(config_data["content"]).decode("utf-8")
                config = yaml.safe_load(content)
                if config and config.get("default_repository") is True:
                    console.print(f"[info]기본 레포지토리 [bold]{repo_name}[/] 발견![/]")
                    return True, repo_name
        except SystemExit:
            continue
    console.print("[info]기존 기본 레포지토리를 찾을 수 없습니다.[/]")
    return False, None


@click.command(name="make-fown-archive")
@click.option(
    "--archive-name",
    "-n",
    default="fown-archive",
    show_default=True,
    help="생성할 아카이브 레포지토리 이름",
)
@click.option(
    "--default",
    is_flag=True,
    default=True,
    help="이 레포지토리를 기본 설정 레포지토리로 지정합니다.",
)
@click.option(
    "--force",
    is_flag=True,
    help="기본 설정 레포지토리가 이미 있어도 강제로 생성합니다.",
)
@click.option(
    "--public",
    is_flag=True,
    help="아카이브 레포지토리를 공개로 설정합니다. 기본값은 비공개입니다.",
)
def make_archive(archive_name: str, default: bool, force: bool, public: bool):
    """저장소 설정을 [bold green]아카이브[/]합니다."""
    current_user = get_github_username()
    if not current_user:
        return

    user_repos = get_user_repositories()

    if default and not force:
        has_default, default_repo = check_existing_default_repo(
            current_user, archive_name, user_repos
        )
        if has_default:
            console.print(
                Panel(
                    f"이미 기본 설정 레포지토리가 존재합니다: [bold]https://github.com/{current_user}/{default_repo}[/]\n"
                    "기존 레포지토리를 계속 사용하거나 --force 옵션을 사용하여 새로 생성하세요.",
                    title="경고",
                    border_style="yellow",
                )
            )
            return

    repo_name = get_available_repo_name(archive_name, user_repos)

    with Progress(
        SpinnerColumn(), TextColumn("[info]아카이브 레포지토리 생성 중...[/]"), transient=True
    ) as progress:
        progress.add_task("", total=None)
        success = create_archive_repo(repo_name, "Fown archive repository", is_public=public)

    if not success:
        return

    # For simplicity, we'll create a default set of labels for the new archive repo.
    # In a real scenario, you might want to fetch them from the current repo.
    labels = [
        Label(name="bug", color="d73a4a", description="Something isn't working"),
        Label(
            name="documentation",
            color="0075ca",
            description="Improvements or additions to documentation",
        ),
    ]

    with Progress(
        SpinnerColumn(), TextColumn("[info]설정 파일 생성 및 푸시 중...[/]"), transient=True
    ) as progress:
        progress.add_task("", total=None)
        success = create_fown_config_files(current_user, repo_name, labels, is_default=default)

    if not success:
        return

    console.print(
        Panel(
            f"아카이브가 생성되었습니다: [bold]https://github.com/{current_user}/{repo_name}[/]"
            + (
                "\n이 레포지토리는 [bold]기본 설정 레포지토리[/]로 지정되었습니다."
                if default
                else ""
            ),
            title="아카이브 완료",
            border_style="green",
        )
    )



"""
레이블 관련 명령어 모듈
"""

import base64
import json
import os
import tempfile
from pathlib import Path
from typing import Dict, List, Optional, Tuple

import rich_click as click
import yaml
from rich.panel import Panel
from rich.progress import Progress
from rich.prompt import Prompt
from rich.table import Table

from fown.core.models.config import Config, Label, Repository
from fown.core.services.github import LabelService
from fown.core.utils.file_io import console, get_git_repo_url, make_github_api_request


@click.group(name="labels")
def labels_group():
    """[bold yellow]레이블[/] 관련 명령어"""
    pass


def get_github_username() -> Optional[str]:
    """Get GitHub username via API."""
    try:
        user_data = make_github_api_request("GET", "user")
        # 명시적 타입 체크 추가
        if isinstance(user_data, dict):
            return user_data.get("login")
        return None
    except SystemExit:
        return None


def find_default_archive_repo() -> Tuple[bool, Optional[str], Optional[str]]:
    """Find the default fown-archive repository."""
    username = get_github_username()
    if not username:
        return False, None, None

    try:
        repos = make_github_api_request("GET", "user/repos", params={"per_page": 100})
        # 명시적 타입 체크 추가
        if not isinstance(repos, list):
            return False, None, None

        for repo in repos:
            if "fown-archive" in repo["name"]:
                try:
                    endpoint = f"repos/{username}/{repo['name']}/contents/.fown/config.yml"
                    config_data = make_github_api_request("GET", endpoint)
                    # 명시적 타입 체크 추가
                    if isinstance(config_data, dict) and "content" in config_data:
                        content = base64.b64decode(config_data["content"]).decode("utf-8")
                        config = yaml.safe_load(content)
                        if config and config.get("default_repository") is True:
                            return True, repo["name"], username
                except SystemExit:
                    continue
    except SystemExit:
        return False, None, None
    return False, None, None


def list_archive_label_files(repo_name: str, owner: str) -> List[Dict]:
    """List label files in the archive repository."""
    try:
        endpoint = f"repos/{owner}/{repo_name}/contents/labels"
        files_data = make_github_api_request("GET", endpoint)
        if isinstance(files_data, list):
            return [
                {"name": item["name"], "path": item["path"], "type": item["type"]}
                for item in files_data
                if item["type"] == "file" and item["name"].endswith(".json")
            ]
        return []
    except SystemExit:
        return []


def get_label_file_content(repo_name: str, owner: str, file_path: str) -> Optional[str]:
    """Get content of a specific label file."""
    try:
        endpoint = f"repos/{owner}/{repo_name}/contents/{file_path}"
        content_data = make_github_api_request("GET", endpoint)
        # 명시적 타입 체크 추가
        if isinstance(content_data, dict) and "content" in content_data:
            content = base64.b64decode(content_data["content"]).decode("utf-8")
            labels_data = json.loads(content)
            with tempfile.NamedTemporaryFile(
                delete=False, suffix=".json", mode="w", encoding="utf-8"
            ) as tf:
                json.dump(labels_data, tf, ensure_ascii=False, indent=2)
                return tf.name
        return None
    except SystemExit:
        return None


def load_labels_from_json(file_path: str) -> List[Label]:
    """Load labels from a JSON file."""
    try:
        with open(file_path, "r", encoding="utf-8") as f:
            data = json.load(f)
        return [Label.from_dict(item) for item in data]
    except (IOError, json.JSONDecodeError) as e:
        console.print(f"[error]레이블 파일 로드 실패:[/] {str(e)}")
        return []


def _display_paginated_menu(
    items: List[Dict],
    current_page: int,
    total_pages: int,
    border_style: str = "cyan",
    title_prefix: str = "",
    columns: Optional[List[Dict[str, str]]] = None,
) -> Table:
    """공통 페이지네이션 메뉴 표시 함수"""
    console.clear()
    console.print(
        Panel(
            f"{title_prefix}레이블 파일 목록 (페이지 {current_page}/{total_pages})",
            border_style=border_style,
        )
    )

    # 기본 컬럼 설정
    if columns is None:
        columns = [
            {"name": "#", "style": "cyan"},
            {"name": "파일명", "style": "green"},
            {"name": "타입", "style": "yellow"},
        ]

    # 테이블 생성
    table = Table(show_header=True)
    for col in columns:
        table.add_column(col["name"], style=col.get("style", ""))

    for i, item in enumerate(items, 1):
        row = [str(i)]
        row.extend(
            [
                item.get("name", "")
                if col["name"].lower() == "파일명"
                else os.path.splitext(item.get("name", ""))[1]
                if col["name"].lower() == "타입"
                else item.get(col["name"].lower(), "")
                for col in columns[1:]
            ]
        )
        table.add_row(*row)

    console.print(table)
    return table


def _handle_pagination_menu(
    files: List[Dict],
    repo_name: str,
    owner: str,
    page_size: int = 5,
    columns: Optional[List[Dict[str, str]]] = None,
) -> Optional[str]:
    """공통 페이지네이션 처리 함수"""
    current_page = 1

    while True:
        # 페이지네이션 계산
        total_pages = (len(files) + page_size - 1) // page_size
        start_index = (current_page - 1) * page_size
        end_index = start_index + page_size
        page_files = files[start_index:end_index]

        # 메뉴 표시
        _display_paginated_menu(
            page_files,
            current_page,
            total_pages,
            border_style="cyan",
            title_prefix="",
            columns=columns,
        )

        # 명령어 안내
        console.print("\n[bold]명령어:[/]")
        if total_pages > 1:
            console.print(f" 1-{len(page_files)}: 레이블 파일 선택")
            console.print(" n: 다음 페이지")
            console.print(" p: 이전 페이지")
        else:
            console.print(f" 1-{len(page_files)}: 레이블 파일 선택")
        console.print(" q: 종료")

        # 사용자 선택 처리
        choice = Prompt.ask("선택").strip().lower()

        # 종료 처리
        if choice == "q":
            return None

        # 페이지 이동 처리
        if choice == "n" and current_page < total_pages:
            current_page += 1
            continue
        elif choice == "p" and current_page > 1:
            current_page -= 1
            continue

        # 선택 검증
        try:
            index = int(choice)
            if 1 <= index <= len(page_files):
                selected_file = page_files[index - 1]
                return get_label_file_content(repo_name, owner, selected_file["path"])
            else:
                console.print("[warning]잘못된 선택입니다.[/warning]")
        except ValueError:
            console.print("[warning]숫자를 입력해주세요.[/warning]")


def load_labels_from_archive(
    repo_name: str, owner: str, show_menu: bool = False
) -> Tuple[List[Label], Optional[str]]:
    """Load labels from an archive repository."""
    labels: List[Label] = []
    temp_file_path: Optional[str] = None

    files = list_archive_label_files(repo_name, owner)
    if not files:
        console.print("[warning]No label files found in the archive.[/warning]")
        return labels, temp_file_path

    if show_menu:
        temp_file_path = _handle_pagination_menu(
            files,
            repo_name,
            owner,
            columns=[
                {"name": "#", "style": "cyan"},
                {"name": "파일명", "style": "green"},
                {"name": "타입", "style": "yellow"},
            ],
        )
    else:
        # 첫 번째 파일 사용
        temp_file_path = get_label_file_content(repo_name, owner, files[0]["path"])

    if temp_file_path:
        labels = load_labels_from_json(temp_file_path)

    return labels, temp_file_path


def apply_labels_to_repo(labels: List[Label], repo_full_name: str) -> int:
    """Apply labels to a repository."""
    success_count = 0
    with Progress() as progress:
        task = progress.add_task("[cyan]레이블 생성 중...[/]", total=len(labels))
        for label in labels:
            if label.name and label.color:
                if LabelService.create_label(label, repo_full_name):
                    success_count += 1
            progress.update(task, advance=1)
    return success_count


@labels_group.command(name="sync")
@click.option("--repo-url", default=None, help="Target GitHub Repository URL.")
@click.option("--labels-file", "-f", default=None, help="Path to labels YAML/JSON file.")
@click.option("--archive", is_flag=True, help="Use labels from an archive repository.")
@click.confirmation_option(prompt="Delete all existing labels and apply new ones?")
def sync_labels(repo_url: Optional[str], labels_file: Optional[str], archive: bool):
    """Synchronize labels by deleting all old ones and applying new ones."""
    if not repo_url:
        repo_url = get_git_repo_url()
    repo = Repository.from_url(repo_url)
    console.print(f"[info]Syncing labels for [bold]{repo.full_name}[/]...[/]")

    labels: List[Label] = []
    temp_file_path: Optional[str] = None

    if labels_file:
        labels = Config.load_labels(labels_file)
    else:
        found, repo_name, owner = find_default_archive_repo()
        if found and repo_name and owner:
            # show_menu 파라미터를 archive 옵션과 연결
            labels, temp_file_path = load_labels_from_archive(repo_name, owner, show_menu=archive)

        # 아카이브에서 레이블을 찾지 못했거나 아카이브 옵션이 없는 경우 기본 레이블 사용
        if not labels:
            console.print("[warning]No labels found in archive, using default.[/warning]")
            default_path = Path(__file__).parent.parent / "data/default_config.yml"
            labels = Config.load_labels(str(default_path))

    if not labels:
        console.print("[error]No labels to apply.[/error]")
        return

    console.print(f"[info]Deleting all existing labels from {repo.full_name}...[/]")
    LabelService.delete_all_labels(repo.full_name)

    console.print(f"[info]Applying {len(labels)} new labels...[/]")
    success_count = apply_labels_to_repo(labels, repo.full_name)

    console.print(
        Panel(f"[success]{success_count}/{len(labels)} labels synced.[/]", title="Complete")
    )

    # 임시 파일 정리
    if temp_file_path:
        os.unlink(temp_file_path)


@labels_group.command(name="clear-all")
@click.option("--repo-url", default=None, help="Target GitHub Repository URL.")
@click.confirmation_option(
    prompt="Are you sure you want to delete all labels? This cannot be undone."
)
def clear_all_labels(repo_url: Optional[str]):
    """Delete all labels from a repository."""
    if not repo_url:
        repo_url = get_git_repo_url()
    repo = Repository.from_url(repo_url)
    console.print(f"[info]Deleting all labels from [bold]{repo.full_name}[/]...[/]")
    LabelService.delete_all_labels(repo.full_name)


@labels_group.command(name="apply")
@click.option("--repo-url", default=None, help="Target GitHub Repository URL.")
@click.option("--labels-file", "-f", required=True, help="Path to labels YAML/JSON file.")
def apply_labels(repo_url: Optional[str], labels_file: str):
    """Create or update labels from a file."""
    if not repo_url:
        repo_url = get_git_repo_url()
    repo = Repository.from_url(repo_url)
    console.print(f"[info]Applying labels to [bold]{repo.full_name}[/]...[/]")

    labels = Config.load_labels(labels_file)
    console.print(f"[info]Loaded {len(labels)} labels.[/]")

    success_count = apply_labels_to_repo(labels, repo.full_name)
    console.print(
        Panel(f"[success]{success_count}/{len(labels)} labels applied.[/]", title="Complete")
    )



"""
파일 입출력 및 유틸리티 함수 모음
"""

import os
import re
import subprocess
from typing import Dict, List, Optional, Tuple, Union

import requests
import yaml
from rich.console import Console
from rich.panel import Panel
from rich.theme import Theme

from fown.cli.auth import load_token

# Rich 설정
theme = Theme(
    {
        "info": "cyan",
        "warning": "yellow",
        "error": "bold red",
        "success": "bold green",
    }
)
console = Console(theme=theme)


def make_github_api_request(
    method: str,
    endpoint: str,
    data: Optional[Dict] = None,
    params: Optional[Dict] = None,
    check_status: bool = True,
) -> Union[Dict, List, None]:
    """GitHub API에 인증된 요청을 보냅니다."""
    token = load_token()
    if not token:
        console.print("[error]로그인이 필요합니다. 'fown auth login'를 실행하세요.[/]")
        raise SystemExit(1)

    headers = {
        "Authorization": f"token {token}",
        "Accept": "application/vnd.github.v3+json",
        "X-GitHub-Api-Version": "2022-11-28",
    }
    url = f"https://api.github.com/{endpoint}"

    try:
        response = requests.request(method, url, headers=headers, json=data, params=params)
        if check_status:
            response.raise_for_status()

        if response.status_code == 204:  # No Content
            return None

        return response.json()
    except requests.exceptions.HTTPError as e:
        if e.response.status_code == 401:
            console.print(
                "[error]인증 실패. 토큰이 유효하지 않거나 만료되었습니다. 'fown auth login'로 다시 로그인하세요.[/]"
            )
        elif e.response.status_code == 404:
            console.print(f"[error]찾을 수 없음: {url}[/]")
        else:
            console.print(f"[error]API 요청 실패 ({e.response.status_code}): {e.response.text}[/]")
        raise SystemExit(1) from e
    except requests.exceptions.RequestException as e:
        console.print(f"[error]네트워크 오류: {e}[/]")
        raise SystemExit(1) from e


def load_yaml(file_path: str) -> Union[List, Dict, None]:
    """YAML 파일 로드"""
    if not os.path.exists(file_path):
        console.print(f"[error]{file_path} 파일이 존재하지 않습니다.[/]")
        raise SystemExit(1)

    with open(file_path, "r", encoding="utf-8") as f:
        return yaml.safe_load(f)


def get_git_repo_url() -> str:
    """현재 디렉터리의 git origin URL 가져오기"""
    try:
        result = subprocess.run(
            ["git", "remote", "get-url", "origin"],
            check=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True,
        )
        return result.stdout.strip()
    except Exception:
        console.print(
            "[error]현재 디렉터리가 git 저장소가 아니거나 origin 원격을 찾을 수 없습니다.[/]"
        )
        raise SystemExit(1)


def extract_repo_info(repo_url: str) -> Tuple[str, str]:
    """GitHub 저장소 URL에서 소유자와 저장소 이름 추출"""
    match = re.match(
        r"(?:https://github\.com/|git@github\.com:)([^/]+)/([^/]+?)(?:\.git)?$", repo_url
    )
    if match:
        owner = match.group(1)
        repo = match.group(2)
        return owner, repo
    else:
        console.print(
            "[error]올바른 GitHub repo URL 형식이 아닙니다.[/]", "예: https://github.com/OWNER/REPO"
        )
        raise SystemExit(1)



"""
스크립트 실행 관련 명령어 모듈
"""

import base64
import json
import os
import shutil  # shutil 모듈 추가
import subprocess
import sys
import tempfile
from pathlib import Path
from typing import Callable, Dict, List, Optional, Tuple

import rich_click as click
import yaml
from rich.panel import Panel
from rich.progress import Progress, SpinnerColumn, TextColumn
from rich.prompt import Prompt
from rich.table import Table

from fown.core.utils.file_io import console, make_github_api_request


@click.group(name="script")
def script_group():
    """[bold yellow]스크립트[/] 관련 명령어"""
    pass


def get_github_username() -> Optional[str]:
    """Get GitHub username via API."""
    try:
        user_data = make_github_api_request("GET", "user")
        # 명시적 타입 체크 추가
        if isinstance(user_data, dict):
            return user_data.get("login")
        return None
    except SystemExit:
        return None


def find_default_archive_repo() -> Tuple[bool, Optional[str], Optional[str]]:
    """Find the default fown-archive repository."""
    username = get_github_username()
    if not username:
        return False, None, None

    try:
        repos = make_github_api_request("GET", "user/repos", params={"per_page": 100})
        if not isinstance(repos, list):
            return False, None, None

        for repo in repos:
            if "fown-archive" in repo["name"]:
                try:
                    endpoint = f"repos/{username}/{repo['name']}/contents/.fown/config.yml"
                    config_data = make_github_api_request("GET", endpoint)
                    if isinstance(config_data, dict) and "content" in config_data:
                        content = base64.b64decode(config_data["content"]).decode("utf-8")
                        config = yaml.safe_load(content)
                        if config and config.get("default_repository") is True:
                            return True, repo["name"], username
                except SystemExit:
                    continue
    except SystemExit:
        return False, None, None
    return False, None, None


def list_archive_script_files(repo_name: str, owner: str) -> List[Dict]:
    """List script files in the archive repository."""
    try:
        endpoint = f"repos/{owner}/{repo_name}/contents/scripts"
        files_data = make_github_api_request("GET", endpoint)
        if isinstance(files_data, list):
            return [
                {
                    "name": item["name"],
                    "path": item["path"],
                    "type": item["type"],
                    "sha": item["sha"],
                }
                for item in files_data
                if item["type"] == "file"
                and (item["name"].endswith(".py") or item["name"].endswith(".sh"))
            ]
        return []
    except SystemExit:
        return []


def get_script_file_content(repo_name: str, owner: str, file_path: str) -> Optional[str]:
    """Get content of a specific script file."""
    try:
        endpoint = f"repos/{owner}/{repo_name}/contents/{file_path}"
        content_data = make_github_api_request("GET", endpoint)
        if isinstance(content_data, dict) and "content" in content_data:
            content = base64.b64decode(content_data["content"]).decode("utf-8")
            with tempfile.NamedTemporaryFile(
                delete=False, suffix=os.path.splitext(file_path)[1], mode="w", encoding="utf-8"
            ) as tf:
                tf.write(content)
                return tf.name
        return None
    except SystemExit:
        return None


def run_script(script_path: str):
    """Executes a script file."""
    console.print(f"[info]Executing script: {script_path}[/]")
    try:
        if script_path.endswith(".py"):
            result = subprocess.run(
                [sys.executable, script_path], capture_output=True, text=True, check=False
            )
        elif script_path.endswith(".sh"):
            # Make script executable
            os.chmod(script_path, 0o755)

            # Windows에서 bash 실행 방식 개선
            if sys.platform == "win32":
                # Git Bash 또는 WSL bash 사용
                bash_paths = [
                    r"C:\Program Files\Git\bin\bash.exe",  # Git Bash
                    r"C:\Windows\System32\bash.exe",  # WSL Bash
                    "/usr/bin/bash",  # Unix-like systems
                ]

                bash_path = next((path for path in bash_paths if os.path.exists(path)), None)

                if bash_path:
                    result = subprocess.run(
                        [bash_path, script_path],
                        capture_output=True,
                        text=True,
                        check=False,
                        env={**os.environ, "MSYS": "winsymlinks:nativestrict"},
                    )
                else:
                    # Fallback to default shell execution
                    result = subprocess.run(
                        ["bash", script_path], capture_output=True, text=True, check=False
                    )
            else:
                # Unix-like systems
                result = subprocess.run(
                    ["bash", script_path], capture_output=True, text=True, check=False
                )
        else:
            console.print(f"[error]Unsupported script type: {script_path}[/error]")
            return

        # 성공적인 실행의 경우 출력만 표시
        if result.stdout.strip():
            console.print(Panel(result.stdout.strip(), title="Script Output", border_style="green"))

        # 에러 출력은 실제 에러가 있는 경우에만 표시
        if result.stderr.strip() and result.returncode != 0:
            console.print(Panel(result.stderr.strip(), title="Script Error", border_style="red"))

    except Exception as e:
        console.print(f"[error]Script execution error: {e}[/error]")
    finally:
        # 스크립트 실행 후 임시 파일 삭제
        if os.path.exists(script_path):
            try:
                os.unlink(script_path)
            except Exception as e:
                console.print(f"[warning]Could not delete temporary script file: {e}[/warning]")


def _display_paginated_menu(
    items: List[Dict],
    current_page: int,
    total_pages: int,
    border_style: str = "cyan",
    title_prefix: str = "",
    columns: Optional[List[Dict[str, str]]] = None,
) -> Table:
    """공통 페이지네이션 메뉴 표시 함수"""
    console.clear()
    console.print(
        Panel(
            f"{title_prefix}스크립트 목록 (페이지 {current_page}/{total_pages})",
            border_style=border_style,
        )
    )

    # 기본 컬럼 설정
    if columns is None:
        columns = [
            {"name": "#", "style": "cyan"},
            {"name": "파일명", "style": "green"},
            {"name": "타입", "style": "yellow"},
        ]

    # 테이블 생성
    table = Table(show_header=True)
    for col in columns:
        table.add_column(col["name"], style=col.get("style", ""))

    for i, item in enumerate(items, 1):
        row = [str(i)]
        row.extend(
            [
                item.get("name", "")
                if col["name"].lower() == "파일명"
                else (
                    os.path.splitext(item.get("name", ""))[1]
                    if col["name"].lower() == "타입"
                    else item.get(col["name"].lower(), "")
                )
                for col in columns[1:]
            ]
        )
        table.add_row(*row)

    console.print(table)
    return table


def _handle_script_pagination_menu(
    files: List[Dict],
    repo_name: str,
    owner: str,
    action_func: Callable[[Dict], None],
    page_size: int = 5,
    columns: Optional[List[Dict[str, str]]] = None,
) -> None:
    """공통 스크립트 페이지네이션 처리 함수"""
    current_page = 1

    while True:
        # 페이지네이션 계산
        total_pages = (len(files) + page_size - 1) // page_size
        start_index = (current_page - 1) * page_size
        end_index = start_index + page_size
        page_files = files[start_index:end_index]

        # 메뉴 표시
        _display_paginated_menu(
            page_files,
            current_page,
            total_pages,
            border_style="cyan",
            title_prefix="",
            columns=columns,
        )

        # 명령어 안내
        console.print("\n[bold]명령어:[/]")
        if total_pages > 1:
            console.print(f" 1-{len(page_files)}: 스크립트 선택")
            console.print(" n: 다음 페이지")
            console.print(" p: 이전 페이지")
        else:
            console.print(f" 1-{len(page_files)}: 스크립트 선택")
        console.print(" q: 종료")

        # 사용자 선택 처리
        choice = Prompt.ask("선택").strip().lower()

        # 종료 처리
        if choice == "q":
            break

        # 페이지 이동 처리
        if choice == "n" and current_page < total_pages:
            current_page += 1
            continue
        elif choice == "p" and current_page > 1:
            current_page -= 1
            continue

        # 선택 검증
        try:
            index = int(choice)
            if 1 <= index <= len(page_files):
                selected_file = page_files[index - 1]
                action_func(selected_file)
                break
            else:
                console.print("[warning]잘못된 선택입니다.[/warning]")
        except ValueError:
            console.print("[warning]숫자를 입력해주세요.[/warning]")


@script_group.command(name="use")
def use_script():
    """Execute a script from the archive repository."""
    found, repo_name, owner = find_default_archive_repo()
    if not found or not repo_name or not owner:
        console.print("[error]Default archive repository not found.[/error]")
        return

    files = list_archive_script_files(repo_name, owner)
    if not files:
        console.print("[warning]No scripts found in the archive.[/warning]")
        return

    def use_action(selected_file: Dict):
        """스크립트 실행 액션 함수"""
        script_path_temp = get_script_file_content(repo_name, owner, selected_file["path"])
        if script_path_temp:
            run_script(script_path_temp)

    _handle_script_pagination_menu(
        files=files, repo_name=repo_name, owner=owner, action_func=use_action
    )


@script_group.command(name="add")
@click.argument("script_path", type=click.Path(exists=True))
def add_script(script_path: str):
    """Add a script to the archive repository."""
    if not (script_path.endswith(".sh") or script_path.endswith(".py")):
        console.print("[error]Only .sh and .py scripts are supported.[/error]")
        return

    found, repo_name, owner = find_default_archive_repo()
    if not found or not repo_name or not owner:
        console.print("[error]Default archive repository not found.[/error]")
        return

    with open(script_path, "rb") as f:
        content = f.read()

    file_name = os.path.basename(script_path)
    endpoint = f"repos/{owner}/{repo_name}/contents/scripts/{file_name}"
    data = {
        "message": f"Add script: {file_name}",
        "content": base64.b64encode(content).decode("utf-8"),
    }

    try:
        make_github_api_request("PUT", endpoint, data=data)
        console.print(f"[success]Script '{file_name}' added successfully.[/success]")
    except SystemExit:
        console.print(f"[error]Failed to add script '{file_name}'.[/error]")


@script_group.command(name="load")
def load_script():
    """Download a script from the archive repository."""
    found, repo_name, owner = find_default_archive_repo()
    if not found or not repo_name or not owner:
        console.print("[error]Default archive repository not found.[/error]")
        return

    files = list_archive_script_files(repo_name, owner)
    if not files:
        console.print("[warning]No scripts found to load.[/warning]")
        return

    def download_action(selected_file: Dict):
        """다운로드 액션 함수"""
        script_path_temp = get_script_file_content(repo_name, owner, selected_file["path"])
        if script_path_temp:
            dest_path = Path.cwd() / selected_file["name"]
            # move the file
            shutil.move(script_path_temp, dest_path)
            console.print(f"[success]Script downloaded to {dest_path}[/success]")

    _handle_script_pagination_menu(
        files=files, repo_name=repo_name, owner=owner, action_func=download_action
    )


@script_group.command(name="delete")
def delete_script():
    """Delete a script from the archive repository."""
    found, repo_name, owner = find_default_archive_repo()
    if not found or not repo_name or not owner:
        console.print("[error]Default archive repository not found.[/error]")
        return

    files = list_archive_script_files(repo_name, owner)
    if not files:
        console.print("[warning]No scripts found to delete.[/warning]")
        return

    def delete_action(selected_file: Dict):
        """스크립트 삭제 액션 함수"""
        if (
            not Prompt.ask(
                f"Are you sure you want to delete '{selected_file['name']}'?", choices=["y", "n"]
            )
            == "y"
        ):
            console.print("Deletion cancelled.")
            return

        endpoint = f"repos/{owner}/{repo_name}/contents/{selected_file['path']}"
        data = {
            "message": f"Delete script: {selected_file['name']}",
            "sha": selected_file["sha"],
        }

        try:
            make_github_api_request("DELETE", endpoint, data=data)
            console.print(f"[success]Script '{selected_file['name']}' deleted.[/success]")
        except SystemExit:
            console.print("[error]Failed to delete script.[/error]")

    _handle_script_pagination_menu(
        files=files,
        repo_name=repo_name,
        owner=owner,
        action_func=delete_action,
        columns=[
            {"name": "#", "style": "cyan"},
            {"name": "파일명", "style": "green"},
            {"name": "타입", "style": "yellow"},
        ],
    )



import configparser
import os
import time
import webbrowser
from pathlib import Path

import click
import requests

# Constants
CLIENT_ID = "Ov23liZ6IrbJN0ISFh8h"
DEVICE_CODE_URL = "https://github.com/login/device/code"
TOKEN_URL = "https://github.com/login/oauth/access_token"
API_BASE_URL = "https://api.github.com"
HEADERS = {"Accept": "application/json"}
SCOPE = "repo"

# Configuration file path
CONFIG_DIR = Path.home() / ".fown"
CONFIG_FILE = CONFIG_DIR / "config.ini"


def ensure_config_dir_exists():
    """Ensures the configuration directory exists."""
    CONFIG_DIR.mkdir(parents=True, exist_ok=True)


def save_token(token):
    """Saves the GitHub token to the config file."""
    ensure_config_dir_exists()
    config = configparser.ConfigParser()
    if CONFIG_FILE.exists():
        config.read(CONFIG_FILE)
    if "github" not in config:
        config["github"] = {}
    config["github"]["token"] = token
    with open(CONFIG_FILE, "w") as configfile:
        config.write(configfile)
    click.echo("Successfully logged in and saved token.")


def load_token():
    """Loads the GitHub token from the config file."""
    if not CONFIG_FILE.exists():
        return None
    config = configparser.ConfigParser()
    config.read(CONFIG_FILE)
    return config.get("github", "token", fallback=None)


def remove_token():
    """Removes the GitHub token from the config file."""
    if not CONFIG_FILE.exists():
        return
    config = configparser.ConfigParser()
    config.read(CONFIG_FILE)
    if "github" in config and "token" in config["github"]:
        del config["github"]["token"]
        with open(CONFIG_FILE, "w") as configfile:
            config.write(configfile)
        click.echo("Successfully logged out.")
    else:
        click.echo("You were not logged in.")


def get_login_status():
    """Checks if the user is logged in and the token is valid."""
    token = load_token()
    if not token:
        return None, "Not logged in. Please run 'fown auth login'."

    headers = {"Authorization": f"token {token}"}
    try:
        response = requests.get(f"{API_BASE_URL}/user", headers=headers)
        response.raise_for_status()
        user_data = response.json()
        return token, f"Logged in as {user_data['login']}."
    except requests.exceptions.RequestException as e:
        if e.response and e.response.status_code == 401:
            return None, "Your token is invalid or has expired. Please log in again."
        return None, f"Failed to verify token: {e}"


@click.group(invoke_without_command=True)
@click.pass_context
def auth(ctx):
    """
    Authenticate with GitHub.

    If no subcommand is specified, shows login status.
    """
    if ctx.invoked_subcommand is None:
        _, message = get_login_status()
        click.echo(message)


@auth.command(name="login")
def start_login():
    """Start the GitHub device login flow."""
    token = load_token()
    if token:
        _, message = get_login_status()
        click.echo(f"You are already logged in. {message}")
        if click.confirm("Do you want to log in again?"):
            pass
        else:
            return

    try:
        response = requests.post(
            DEVICE_CODE_URL,
            headers=HEADERS,
            data={"client_id": CLIENT_ID, "scope": SCOPE},
        )
        response.raise_for_status()
        resp_data = response.json()
    except requests.exceptions.RequestException as e:
        click.echo(f"Error requesting device code: {e}", err=True)
        return

    user_code = resp_data["user_code"]
    device_code = resp_data["device_code"]
    verification_uri = resp_data["verification_uri"]
    expires_in = resp_data["expires_in"]
    interval = resp_data.get("interval", 5)

    click.echo(f"Your one-time code is: {click.style(user_code, fg='yellow', bold=True)}")
    if click.confirm("Press Enter to open a browser to the verification page...", default=True):
        webbrowser.open(verification_uri)
        click.echo(f"If the browser does not open, please visit: {verification_uri}")

    total_wait = 0
    with click.progressbar(length=expires_in, label="Waiting for authorization...") as bar:
        while total_wait < expires_in:
            time.sleep(interval)
            total_wait += interval
            bar.update(interval)

            try:
                poll_resp = requests.post(
                    TOKEN_URL,
                    headers=HEADERS,
                    data={
                        "client_id": CLIENT_ID,
                        "device_code": device_code,
                        "grant_type": "urn:ietf:params:oauth:grant-type:device_code",
                    },
                )
                poll_resp.raise_for_status()
                token_data = poll_resp.json()
            except requests.exceptions.RequestException:
                # Don't fail on polling errors, just continue
                continue

            if "access_token" in token_data:
                save_token(token_data["access_token"])
                return
            elif token_data.get("error") == "slow_down":
                interval += 5  # Increase interval if GitHub asks us to slow down
            elif token_data.get("error") not in ["authorization_pending"]:
                click.echo(
                    f"\nLogin failed or canceled: {token_data.get('error_description', 'Unknown error')}",
                    err=True,
                )
                return

    click.echo("\nLogin timed out. Please try again.", err=True)


@auth.command(name="logout")
def logout():
    """Log out from GitHub and remove the stored token."""
    remove_token()


@auth.command(name="status")
def login_status():
    """Check GitHub authentication status."""
    _, message = get_login_status()
    click.echo(message)



"""
파일 관련 명령어 모듈
"""

import base64
import json
import os
from pathlib import Path
from typing import Callable, Dict, List, Literal, Optional, Tuple, Union

import rich_click as click
import yaml
from rich.panel import Panel
from rich.progress import Progress, SpinnerColumn, TextColumn
from rich.prompt import Prompt
from rich.table import Table

from fown.core.services.github import get_github_username, get_user_repositories
from fown.core.utils.file_io import console, make_github_api_request


@click.group(name="file")
def file_group():
    """[bold yellow]파일[/] 관련 명령어

    아카이브 레포지토리의 파일을 관리합니다.
    """
    pass


def find_default_archive_repo() -> Tuple[bool, Optional[str], Optional[str]]:
    """기본 아카이브 레포지토리 찾기"""
    username = get_github_username()
    if not username:
        console.print("[error]GitHub 사용자 정보를 가져올 수 없습니다.[/]")
        return False, None, None

    repos = get_user_repositories()
    repo_names = {repo["name"] for repo in repos}

    for i in range(10):
        suffix = "" if i == 0 else str(i)
        repo_name = f"fown-archive{suffix}"

        if repo_name in repo_names:
            try:
                endpoint = f"repos/{username}/{repo_name}/contents/.fown/config.yml"
                content_data = make_github_api_request("GET", endpoint)
                if isinstance(content_data, dict) and "content" in content_data:
                    content = base64.b64decode(content_data["content"]).decode("utf-8")
                    config = yaml.safe_load(content)
                    if config and config.get("default_repository") is True:
                        return True, repo_name, username
            except SystemExit:
                continue  # If repo exists but config is not found, continue

    return False, None, None


def list_archive_files(repo_name: str, owner: str, path: str = "files") -> List[Dict]:
    """아카이브 레포지토리의 특정 경로에 있는 파일/폴더 목록 가져오기"""
    try:
        endpoint = f"repos/{owner}/{repo_name}/contents/{path}"
        files_data = make_github_api_request("GET", endpoint)
        if isinstance(files_data, list):
            return [
                {
                    "name": item["name"],
                    "path": item["path"],
                    "type": item["type"],
                    "sha": item["sha"],
                }
                for item in files_data
            ]
        return []
    except SystemExit:
        return []


@file_group.command(name="add")
@click.argument("path", type=click.Path(exists=True))
def add_file(path: str):
    """아카이브 레포지토리에 [bold green]파일 또는 폴더를 추가[/]합니다."""
    found, repo_name, owner = find_default_archive_repo()
    if not found or not repo_name or not owner:
        console.print("[error]기본 아카이브 레포지토리를 찾을 수 없습니다.[/]")
        return

    path_obj = Path(path)
    if path_obj.is_dir():
        upload_directory(path_obj, owner, repo_name)
    else:
        upload_file(path_obj, owner, repo_name)


def upload_file(file_path: Path, owner: str, repo_name: str, base_repo_path: str = "files"):
    """지정된 파일을 아카이브 레포지토리에 업로드합니다."""
    try:
        with open(file_path, "rb") as f:
            content_bytes = f.read()
        content_base64 = base64.b64encode(content_bytes).decode("utf-8")

        repo_file_path = f"{base_repo_path}/{file_path.name}"
        endpoint = f"repos/{owner}/{repo_name}/contents/{repo_file_path}"

        # 기존 파일 존재 여부 확인
        try:
            existing_file = make_github_api_request("GET", endpoint)
        except SystemExit:
            existing_file = None

        # 파일 존재 시 처리
        if existing_file and isinstance(existing_file, dict):
            # 사용자에게 선택지 제공
            console.print(f"[yellow]파일 '{file_path.name}'이(가) 이미 존재합니다.[/]")
            console.print("[bold]작업을 선택하세요:[/]")
            console.print(" [bold green]o[/] 덮어쓰기")
            console.print(" [bold green]s[/] 새 이름으로 저장")
            console.print(" [bold green]n[/] 취소")

            action = Prompt.ask("선택", choices=["o", "s", "n"], default="n")

            if action == "n":
                console.print("[info]파일 업로드가 취소되었습니다.[/]")
                return

            if action == "s":
                # 새 이름 생성 (파일명_숫자.확장자)
                base_name = file_path.stem
                ext = file_path.suffix
                counter = 1
                while True:
                    new_filename = f"{base_name}_{counter}{ext}"
                    new_repo_path = f"{base_repo_path}/{new_filename}"
                    try:
                        make_github_api_request(
                            "GET", f"repos/{owner}/{repo_name}/contents/{new_repo_path}"
                        )
                        counter += 1
                    except SystemExit:
                        # 새 파일명으로 경로 업데이트
                        repo_file_path = new_repo_path
                        break

        with Progress(
            SpinnerColumn(), TextColumn("[progress.description]{task.description}"), console=console
        ) as progress:
            progress.add_task(
                description=f"[cyan]파일 업로드 중: {Path(repo_file_path).name}[/]", total=None
            )

            payload = {
                "message": f"Add file: {Path(repo_file_path).name}",
                "content": content_base64,
                "branch": "main",
            }

            # 기존 파일 덮어쓰기의 경우 SHA 추가
            if existing_file and isinstance(existing_file, dict) and "sha" in existing_file:
                payload["sha"] = existing_file["sha"]

            make_github_api_request(
                "PUT", f"repos/{owner}/{repo_name}/contents/{repo_file_path}", data=payload
            )
            console.print(
                f"[success]파일 [bold]{Path(repo_file_path).name}[/]이(가) 성공적으로 업로드되었습니다![/]"
            )

    except (SystemExit, Exception) as e:
        console.print(f"[error]파일 업로드 중 오류 발생:[/] {str(e)}")


def upload_directory(dir_path: Path, owner: str, repo_name: str) -> None:
    """지정된 디렉토리의 모든 파일을 아카이브 레포지토리에 업로드합니다."""
    console.print(f"[info]디렉토리 업로드 시작: [bold]{dir_path.name}[/][/]")
    for root, _, files in os.walk(dir_path):
        for file in files:
            local_path = Path(root) / file
            relative_path = local_path.relative_to(dir_path)
            repo_path = f"files/{dir_path.name}/{relative_path}"
            endpoint = f"repos/{owner}/{repo_name}/contents/{repo_path}"

            try:
                with open(local_path, "rb") as f:
                    content_bytes = f.read()
                content_base64 = base64.b64encode(content_bytes).decode("utf-8")

                with Progress(
                    SpinnerColumn(),
                    TextColumn("[progress.description]{task.description}"),
                    console=console,
                ) as progress:
                    progress.add_task(
                        description=f"[cyan]파일 업로드 중: {local_path.name}[/]", total=None
                    )
                    payload = {
                        "message": f"Add file: {local_path.name}",
                        "content": content_base64,
                        "branch": "main",
                    }
                    make_github_api_request("PUT", endpoint, data=payload)
                    console.print(
                        f"[success]파일 [bold]{local_path.name}[/]이(가) 성공적으로 업로드되었습니다![/]"
                    )

            except (SystemExit, Exception) as e:
                console.print(f"[error]파일 '{local_path.name}' 업로드 중 오류 발생:[/] {str(e)}")


@file_group.command(name="load")
def load_file():
    """아카이브 레포지토리의 [bold green]파일 또는 폴더를 다운로드[/]합니다."""
    found, repo_name, owner = find_default_archive_repo()
    if not found or not repo_name or not owner:
        console.print("[error]기본 아카이브 레포지토리를 찾을 수 없습니다.[/]")
        return

    navigate_and_download(owner, repo_name, "files")


def _process_download_selection(
    owner: str, repo_name: str, current_path: str, selected_item: Dict
) -> Optional[str]:
    """선택된 항목에 대한 다운로드 처리를 수행합니다."""
    if selected_item["type"] == "dir":
        # 디렉토리 다운로드 처리
        action = Prompt.ask(
            f"'{selected_item['name']}'은(는) 디렉토리입니다. 전체 다운로드(y) 또는 폴더 진입(n) 중 선택하세요.",
            choices=["y", "n"],
            default="y",
        )
        if action == "y":
            download_directory(owner, repo_name, selected_item["path"])
            return None  # 다운로드 후 종료
        else:
            return selected_item["path"]  # 폴더 진입
    else:
        # 파일 다운로드
        download_item(owner, repo_name, selected_item)
        return None  # 다운로드 후 종료


def _display_paginated_menu(
    items: List[Dict],
    current_path: str,
    current_page: int,
    total_pages: int,
    border_style: str = "cyan",
    title_prefix: str = "",
    columns: Optional[List[Dict[str, str]]] = None,
) -> Table:
    """공통 페이지네이션 메뉴 표시 함수"""
    console.clear()
    console.print(
        Panel(
            f"{title_prefix}[bold]{current_path}[/] 경로의 내용 (페이지 {current_page}/{total_pages})",
            border_style=border_style,
        )
    )

    # 기본 컬럼 설정
    if columns is None:
        columns = [
            {"name": "#", "style": "cyan"},
            {"name": "이름", "style": "green"},
            {"name": "타입", "style": "yellow"},
        ]

    # 테이블 생성
    table = Table(show_header=True)
    for col in columns:
        table.add_column(col["name"], style=col.get("style", ""))

    for i, item in enumerate(items, 1):
        row = [str(i)]
        row.extend(
            [
                item.get("name", "")
                if col["name"].lower() == "이름"
                else item.get("type", "")
                if col["name"].lower() == "타입"
                else item.get(col["name"].lower(), "")
                for col in columns[1:]
            ]
        )
        table.add_row(*row)

    console.print(table)
    return table


def _validate_page_choice(
    choice: str, page_files: List[Dict], current_page: int, total_pages: int
) -> Optional[Union[Literal["back", "next", "prev", "retry"], int]]:
    """페이지 선택을 검증하고 처리합니다."""
    # 종료 처리
    if choice == "q":
        return None

    # 뒤로가기 처리
    if choice == "b":
        return "back"

    # 페이지 이동 처리
    if choice == "n" and current_page < total_pages:
        return "next"
    elif choice == "p" and current_page > 1:
        return "prev"

    # 항목 선택 처리
    try:
        index = int(choice)
        if 1 <= index <= len(page_files):
            return index
        console.print("[warning]잘못된 선택입니다.[/warning]")
    except ValueError:
        console.print("[warning]숫자를 입력해주세요.[/warning]")

    return "retry"


def _display_pagination_menu_options(total_pages: int, page_files_length: int) -> None:
    """페이지네이션 메뉴 옵션을 표시합니다."""
    console.print("\n[bold]명령어:[/]")
    if total_pages > 1:
        console.print(f" 1-{page_files_length}: 항목 선택")
        console.print(" n: 다음 페이지")
        console.print(" p: 이전 페이지")
    else:
        console.print(f" 1-{page_files_length}: 항목 선택")
    console.print(" b: 뒤로가기")
    console.print(" q: 종료")


def _process_pagination_choice(
    choice: str,
    page_files: List[Dict],
    current_page: int,
    total_pages: int,
    process_selection_func: Callable,
    owner: str,
    repo_name: str,
    current_path: str,
) -> Optional[str]:
    """페이지네이션 선택을 처리합니다."""
    # 선택 검증 및 처리
    validation_result = _validate_page_choice(choice, page_files, current_page, total_pages)

    # 결과에 따른 처리
    if validation_result is None:
        return None
    elif validation_result == "back":
        if current_path == "files":
            return None
        return str(Path(current_path).parent)
    elif validation_result == "next":
        return "next_page"
    elif validation_result == "prev":
        return "prev_page"
    elif validation_result == "retry":
        return "retry"

    # 항목 선택 처리
    selected_item = page_files[validation_result - 1]
    result = process_selection_func(owner, repo_name, current_path, selected_item)

    # 결과에 따른 처리
    if result is None:
        return None
    return result


def _handle_pagination_menu(
    files: List[Dict],
    process_selection_func: Callable,
    repo_name: str,
    owner: str,
    current_path: str,
    page_size: int = 5,
    border_style: str = "cyan",
    title_prefix: str = "",
    columns: Optional[List[Dict[str, str]]] = None,
) -> Optional[str]:
    """공통 페이지네이션 처리 함수"""
    current_page = 1

    while True:
        # 파일/폴더 목록 가져오기
        items = list_archive_files(repo_name, owner, current_path)
        if not items:
            console.print("[warning]파일이나 폴더가 없습니다.[/]")
            return None

        # 페이지네이션 계산
        total_pages = (len(items) + page_size - 1) // page_size
        start_index = (current_page - 1) * page_size
        end_index = start_index + page_size
        page_files = items[start_index:end_index]

        # 메뉴 표시
        _display_paginated_menu(
            page_files, current_path, current_page, total_pages, border_style, title_prefix, columns
        )

        # 명령어 안내
        _display_pagination_menu_options(total_pages, len(page_files))

        # 사용자 선택 처리
        choice = Prompt.ask("선택").strip().lower()

        # 선택 처리
        result = _process_pagination_choice(
            choice,
            page_files,
            current_page,
            total_pages,
            process_selection_func,
            owner,
            repo_name,
            current_path,
        )

        # 결과에 따른 처리
        if result is None:
            return None
        elif result == "next_page":
            current_page += 1
            continue
        elif result == "prev_page":
            current_page -= 1
            continue
        elif result == "retry":
            continue

        current_path = result
        current_page = 1  # 새로운 경로로 이동 시 첫 페이지로 리셋


def navigate_and_download(owner: str, repo_name: str, current_path: str):
    """파일/폴더를 탐색하고 다운로드/뒤로가기 옵션을 제공합니다."""
    # 파일/폴더 목록 가져오기
    items = list_archive_files(repo_name, owner, current_path)
    if not items:
        console.print("[warning]파일이나 폴더가 없습니다.[/]")
        return

    # 페이지네이션 및 다운로드 처리
    _handle_pagination_menu(
        files=items,
        process_selection_func=_process_download_selection,
        repo_name=repo_name,
        owner=owner,
        current_path=current_path,
        border_style="cyan",
        title_prefix="레포지토리의 파일 목록 ",
    )


def download_item(owner: str, repo_name: str, item: Dict):
    """선택된 파일 또는 폴더를 다운로드합니다."""
    if item["type"] == "file":
        download_single_file(owner, repo_name, item)
    elif item["type"] == "dir":
        download_directory(owner, repo_name, item["path"])


def download_single_file(owner: str, repo_name: str, file_item: Dict, local_dir: Path = Path.cwd()):
    """단일 파일을 다운로드합니다."""
    try:
        endpoint = f"repos/{owner}/{repo_name}/contents/{file_item['path']}"
        content_data = make_github_api_request("GET", endpoint)
        # 명시적 타입 체크 추가
        if isinstance(content_data, dict) and "content" in content_data:
            content = base64.b64decode(content_data["content"])
            local_path = local_dir / file_item["name"]
            local_path.parent.mkdir(parents=True, exist_ok=True)

            if local_path.exists():
                if (
                    not Prompt.ask(
                        f"[yellow]파일 '{local_path}'이(가) 이미 존재합니다. 덮어쓰시겠습니까?[/]",
                        choices=["y", "n"],
                        default="n",
                    )
                    == "y"
                ):
                    console.print("[info]다운로드가 취소되었습니다.[/]")
                    return

            with open(local_path, "wb") as f:
                f.write(content)
            console.print(f"[success]파일 [bold]{file_item['name']}[/] 다운로드 완료![/]")
        else:
            console.print(
                f"[warning]파일 [bold]{file_item['name']}[/]의 내용을 가져올 수 없습니다.[/]"
            )

    except (SystemExit, Exception) as e:
        console.print(f"[error]파일 다운로드 실패:[/] {e}")


def download_directory(owner: str, repo_name: str, dir_path: str):
    """디렉토리 내용을 재귀적으로 다운로드합니다."""
    console.print(f"[info]디렉토리 다운로드 시작: [bold]{dir_path}[/][/]")
    items = list_archive_files(repo_name, owner, dir_path)

    # 명시적 타입 체크 추가
    if not isinstance(items, list):
        console.print(f"[warning]디렉토리 [bold]{dir_path}[/]의 내용을 가져올 수 없습니다.[/]")
        return

    local_dir_name = Path(dir_path).name
    local_base_dir = Path.cwd() / local_dir_name

    for item in items:
        if item["type"] == "file":
            download_single_file(owner, repo_name, item, local_base_dir)
        elif item["type"] == "dir":
            download_directory(owner, repo_name, item["path"])


@file_group.command(name="delete")
def delete_file():
    """아카이브 레포지토리의 [bold red]파일 또는 폴더를 삭제[/]합니다."""
    found, repo_name, owner = find_default_archive_repo()
    if not found or not repo_name or not owner:
        console.print("[error]기본 아카이브 레포지토리를 찾을 수 없습니다.[/]")
        return

    navigate_and_delete(owner, repo_name, "files")


def _process_delete_selection(
    owner: str, repo_name: str, current_path: str, selected_item: Dict
) -> Optional[str]:
    """선택된 항목에 대한 삭제 처리를 수행합니다."""
    if selected_item["type"] == "dir":
        # 디렉토리 삭제 처리
        action = Prompt.ask(
            f"'{selected_item['name']}'은(는) 디렉토리입니다. 전체 삭제(y) 또는 폴더 진입(n) 중 선택하세요.",
            choices=["y", "n"],
            default="y",
        )
        if action == "y":
            # 디렉토리 삭제 확인
            confirm = Prompt.ask(
                f"[bold red]정말로 '{selected_item['name']}' 디렉토리와 모든 하위 파일을 삭제하시겠습니까?[/]",
                choices=["y", "n"],
                default="n",
            )
            if confirm == "y":
                delete_directory_recursive(owner, repo_name, selected_item["path"])
                return current_path  # 현재 경로 유지하여 목록 새로고침
            return current_path
        else:
            return selected_item["path"]  # 폴더 진입
    else:
        # 파일 삭제 처리
        confirm = Prompt.ask(
            f"[bold red]정말로 '{selected_item['name']}'을(를) 삭제하시겠습니까?[/]",
            choices=["y", "n"],
            default="n",
        )
        if confirm == "y":
            delete_single_file(owner, repo_name, selected_item)
            return current_path  # 현재 경로 유지하여 목록 새로고침
        return current_path


def navigate_and_delete(owner: str, repo_name: str, current_path: str):
    """파일/폴더를 탐색하고 삭제/뒤로가기 옵션을 제공합니다."""
    # 페이지네이션 및 삭제 처리
    _handle_pagination_menu(
        files=list_archive_files(repo_name, owner, current_path),
        process_selection_func=_process_delete_selection,
        repo_name=repo_name,
        owner=owner,
        current_path=current_path,
        border_style="red",
        title_prefix="경로의 내용 ",
    )


def delete_single_file(owner: str, repo_name: str, file_item: Dict):
    """단일 파일을 삭제합니다."""
    try:
        endpoint = f"repos/{owner}/{repo_name}/contents/{file_item['path']}"
        payload = {
            "message": f"Delete file: {file_item['name']}",
            "sha": file_item["sha"],
            "branch": "main",
        }
        make_github_api_request("DELETE", endpoint, data=payload)
        console.print(f"[success]파일 [bold]{file_item['name']}[/] 삭제 완료![/]")
    except (SystemExit, Exception) as e:
        console.print(f"[error]파일 삭제 중 오류 발생:[/] {e}")


def delete_directory_recursive(owner: str, repo_name: str, dir_path: str):
    """디렉토리와 그 내용을 재귀적으로 삭제합니다."""
    items = list_archive_files(repo_name, owner, dir_path)
    for item in items:
        if item["type"] == "dir":
            delete_directory_recursive(owner, repo_name, item["path"])
        else:
            delete_single_file(owner, repo_name, item)

    console.print(f"[success]디렉토리 [bold]{Path(dir_path).name}[/]의 내용이 삭제되었습니다.[/]")


if __name__ == "__main__":
    file_group()
